#define VIRTUAL_BASE ((char*) 42)  \

/*2:*/
#line 120 "./dirty.w"

#include <config.h>
#include "lkconfig.h"
/*3:*/
#line 136 "./dirty.w"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>

/*:3*/
#line 123 "./dirty.w"

#include "error.h"
#include "memory.h"
#include "dirty.h"

/*31:*/
#line 502 "./dirty.w"

static dirty_queue_node_t sentinel;


/*:31*//*37:*/
#line 581 "./dirty.w"

#if DIRTY_SET == DIRTY_SET_FIFO
static int num_queues= 0;
#endif

/*:37*/
#line 128 "./dirty.w"

/*18:*/
#line 370 "./dirty.w"

#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
static int cmp_virtual_char(const void*a,const void*b);
static int
cmp_virtual_char(const void*a,const void*b)
{
return(const char*)a-(const char*)b;
}

static void prn_virtual_char(void*a);
static void
prn_virtual_char(void*a)
{
printf("%d",(const char*)a-VIRTUAL_BASE);
}
#endif

/*:18*/
#line 129 "./dirty.w"

/*7:*/
#line 217 "./dirty.w"

dirty_set_t*
dirty_create(int n,int full,int seed)
{
dirty_set_t*ds= new_of(dirty_set_t);
errorif(n<0,"Dirty set must be over non-empty range: n=%d",n);
ds->n= n;
ds->num_used= -1;
#if DIRTY_SET==DIRTY_SET_SPLAY_ROOT
/*17:*/
#line 346 "./dirty.w"

ds->dict= dict_create(cmp_virtual_char,prn_virtual_char);


/*:17*/
#line 226 "./dirty.w"

#else
/*27:*/
#line 472 "./dirty.w"

ds->queue= new_arr_of(dirty_queue_node_t,n);

/*:27*//*36:*/
#line 573 "./dirty.w"

ds->prng= prng_new(PRNG_DEFAULT,seed^21264^num_queues++);

/*:36*/
#line 228 "./dirty.w"

#endif
if(full){dirty_make_full(ds,seed);}
else{dirty_make_empty(ds);}
return ds;
}


/*:7*//*8:*/
#line 237 "./dirty.w"

void
dirty_destroy(dirty_set_t*ds)
{
if(ds){
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*19:*/
#line 388 "./dirty.w"

if(ds->dict)dict_destroy(ds->dict,NULL);


/*:19*/
#line 243 "./dirty.w"

#else
/*28:*/
#line 476 "./dirty.w"

free_mem(ds->queue);mem_deduct(ds->n*sizeof(dirty_queue_node_t));
ds->head= NULL;
ds->tail= NULL;

/*:28*//*38:*/
#line 587 "./dirty.w"

prng_free(ds->prng);

/*:38*/
#line 245 "./dirty.w"

#endif
ds->n= -1;
free_mem(ds);mem_deduct(sizeof(dirty_set_t));
}
}


/*:8*//*9:*/
#line 254 "./dirty.w"

void
dirty_add(dirty_set_t*ds,const int v)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*22:*/
#line 412 "./dirty.w"

if(!dict_insert(ds->dict,VIRTUAL_BASE+v))ds->num_used++;

/*:22*/
#line 259 "./dirty.w"

#else
/*30:*/
#line 487 "./dirty.w"

{dirty_queue_node_t*dq= ds->queue;
if(dq[v].next==NULL){
dq[v].next= &sentinel;
if(ds->tail)ds->tail->next= dq+v;
else ds->head= dq+v;
ds->tail= dq+v;
ds->num_used++;
}
}

/*:30*/
#line 261 "./dirty.w"

#endif
}

/*:9*//*10:*/
#line 266 "./dirty.w"

int
dirty_includes(dirty_set_t*ds,const int v)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*23:*/
#line 420 "./dirty.w"

return dict_find(ds->dict,VIRTUAL_BASE+v)!=NULL;

/*:23*/
#line 271 "./dirty.w"

#else
/*29:*/
#line 482 "./dirty.w"

return ds->queue[v].next!=NULL;

/*:29*/
#line 273 "./dirty.w"

#endif
}


/*:10*//*11:*/
#line 281 "./dirty.w"

int
dirty_remove(dirty_set_t*ds)
{
int return_value= -1;
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*24:*/
#line 427 "./dirty.w"

{void*vp= dict_delete_any(ds->dict,NULL);
if(vp){ds->num_used--;return((char*)vp)-VIRTUAL_BASE;}
}


/*:24*/
#line 287 "./dirty.w"

#else
/*32:*/
#line 510 "./dirty.w"

if(ds->head){
dirty_queue_node_t*old_head= ds->head;
return_value= old_head-ds->queue;
if(old_head==ds->tail)ds->tail= ds->head= NULL;

else ds->head= old_head->next;
old_head->next= NULL;
ds->num_used--;
}

/*:32*/
#line 289 "./dirty.w"

#endif
return return_value;
}


/*:11*//*12:*/
#line 296 "./dirty.w"

void
dirty_make_full(dirty_set_t*ds,int seed)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*20:*/
#line 395 "./dirty.w"

{
int i;
dict_t*d= ds->dict;
for(i= 0;i<n;i++){
dict_insert(d,VIRTUAL_BASE+i);
}
}

/*:20*/
#line 301 "./dirty.w"

#else
/*39:*/
#line 593 "./dirty.w"

{
const int n= ds->n;
prng_t*prng= ds->prng;
int i,*work= new_arr_of(int,n);

/*40:*/
#line 607 "./dirty.w"

for(i= 0;i<n;i++)work[i]= i;
for(i= 0;i<n;i++){
const int next= prng_unif_int(prng,n-i);
const int t= work[next];
work[next]= work[n-1-i];
work[n-1-i]= t;
}

/*:40*/
#line 599 "./dirty.w"

/*41:*/
#line 617 "./dirty.w"

{
dirty_queue_node_t*q= ds->queue;
ds->head= q+work[0];
ds->tail= q+work[n-1];
for(i= 0;i<n-1;i++){
q[work[i]].next= q+work[i+1];
}
q[work[n-1]].next= &sentinel;
}



/*:41*/
#line 600 "./dirty.w"


free_mem(work);mem_deduct(n*sizeof(int));
}


/*:39*/
#line 303 "./dirty.w"

#endif
ds->num_used= ds->n;
}

/*:12*//*13:*/
#line 309 "./dirty.w"

void
dirty_make_empty(dirty_set_t*ds)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
/*21:*/
#line 406 "./dirty.w"

dict_delete_all(ds->dict,NULL);

/*:21*/
#line 314 "./dirty.w"

#else
/*33:*/
#line 539 "./dirty.w"

if(ds->num_used<0||ds->num_used>=ds->n/2){
ds->head= ds->tail= NULL;
{int i;
const int n= ds->n;
dirty_queue_node_t*dq= ds->queue;
for(i= 0;i<n;i++)dq[i].next= NULL;
}
}else{
int return_value;
while(ds->head){/*32:*/
#line 510 "./dirty.w"

if(ds->head){
dirty_queue_node_t*old_head= ds->head;
return_value= old_head-ds->queue;
if(old_head==ds->tail)ds->tail= ds->head= NULL;

else ds->head= old_head->next;
old_head->next= NULL;
ds->num_used--;
}

/*:32*/
#line 549 "./dirty.w"
}
}

/*:33*/
#line 316 "./dirty.w"

#endif
ds->num_used= 0;
}


/*:13*//*14:*/
#line 323 "./dirty.w"

int
dirty_has_elements(dirty_set_t*ds)
{
return ds->num_used>0;
}



/*:14*/
#line 130 "./dirty.w"

const char*dirty_rcs_id= "$Id: dirty.w,v 1.13 1998/10/17 22:06:19 neto Exp neto $";

/*:2*/
