
\noindent Copyright \copyright 1994, 1995, 1996, 1997, 1998 David Neto
\smallskip

\noindent 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
\smallskip

\noindent 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
\smallskip

\noindent   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.
\smallskip

\noindent   
   You may contact David Neto via email at {\tt netod@@acm.org}, or with
   greater latency at
\smallskip
\noindent{\obeylines
     Department of Computer Science
     University of Toronto
     10 King's College Rd.
     Toronto, Ontario
     M5S 3G4
     Canada
}
\medskip


\noindent\hbox{}\hrule\hbox{}\penalty-1000
\vskip0.5cm\relax



@i webdefs.w
@i types.w

{\obeylines
$Log: dirty.w,v $
Revision 1.13  1998/10/17 22:06:19  neto
Missed the action parameter on dict delete any

Revision 1.12  1998/08/20 17:03:04  neto
Make a better fit between FIFO remove and FIFO make empty.
Mainly, don't return from the function within FIFO remove.  That was a bug.

Revision 1.11  1998/08/20 15:54:13  neto
Save on lots of function calls in FIFO make empty.

Revision 1.10  1998/08/15 16:07:17  neto
Made a more efficient remove in the queue array case.
Now have dirty includes query.

Revision 1.9  1998/08/07 23:31:29  neto
Had two decls for dirty rcs id.

Revision 1.8  1998/08/07 22:58:41  neto
Export the rcs id.

Revision 1.7  1998/08/07 22:00:17  neto
Edited after seeing its DVI output.
Added a test suite, program dirtytest

Revision 1.6  1998/08/07 16:15:36  neto
Removed consts.
Added .next accessors.
Fied CWEBisms

Revision 1.5  1998/08/07 15:56:03  neto
Fixed the rcsid
Fixed the spurious hash endif
Better documentation of headers.

Revision 1.4  1998/08/07 15:29:07  neto
Finished implmentation of queue part.
Need to compile and TeX.

Revision 1.3  1998/08/07 02:01:45  neto
Fixed some TeX.
Still unbalanced endif.  Huh>

Revision 1.2  1998/08/07 01:55:22  neto
Fixed CWEB header stuff.

Revision 1.1  1998/08/07 01:53:39  neto
Initial revision

}

@*Dirty sets.
This module is implements the sets of integers used as ``dirty'' sets,
also known as ``active'' sets.  Each set covers a universe of $0\ldots n-1$,
where $n$ is known in advance.

I used to use only a splay-tree based dictionary
where removal was just removing the root.  But the implementations
by Johnson, Bentley,
McGeoch and Rothberg,  and by Applegate, Bixby, Chvatal, and Cook
use FIFO queues.

So I've factored the dirty set implementation out from
modules \module{JBMR} and \module{MATCH} for easier maintenance.



@s dirty_set_t int

@ The outline of this module is as follows:
@c
#include <config.h>
#include "lkconfig.h"
@<System headers@>@;
#include "error.h"
#include "memory.h"
#include "dirty.h"

@<Module variables@>@;
@<Module subroutines@>@;
@<Subroutines@>@;
const char *dirty_rcs_id = "$Id: dirty.w,v 1.13 1998/10/17 22:06:19 neto Exp neto $";

@ We will be using many routines from external libraries.  The interfaces
to those routines are described in the following headers.

@<System headers@>=
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>

@ The exported interface is contained in the \file{dirty.h} header file,
which has the following form.

@(dirty.h@>=
#if !defined(_DIRTY_H_)
#define _DIRTY_H_
extern const char *dirty_rcs_id;
@<Module headers required for the interface@>@;
@<Exported early type definitions@>@;
@<Exported type definitions@>@;
@<Exported subroutines@>@;
#endif

@ The interface is as follows:

|dirty_set_t *dirty_create(int n, int full, int seed)| creates 
a dirty set over the universe
$0\ldots n-1$.  If |full| is non-zero, then the set is created with all 
the elements present, and otherwise, it is created empty.
The |seed| parameter is used to fill the set.  See below.

|void dirty_make_full(dirty_set_t *ds, int seed)| fills the set with all
the elements in the universe.  If this is done randomly, then use
|seed| as the seed, with appropriate salt.  (Vague, huh?  Well, what
it does depends on which implementation is used.)

|void dirty_make_empty(dirty_set_t *ds)| removes all the
elements from the set.

|int dirty_has_elements(dirty_set_t *ds)| answers a non-zero value 
if the set is non-empty.

|void dirty_add(dirty_set_t *ds, const int v)| adds $v$ to dirty set |ds| if
it isn't already a memeber of the set.

|int dirty_remove(dirty_set_t *ds)| removes and answers some member from
set |ds|.  If set |ds| is empty, then answer $-1$.

|int dirty_includes(dirty_set_t *ds, int v)| is non-negative if and only
if set |ds| includes value |v|.

|void dirty_destroy(dirty_set_t *ds)| destroys a dirty set.


@<Exported subroutines@>=
dirty_set_t *dirty_create(int n, int full, int seed);
void dirty_make_full(dirty_set_t *ds, int seed);
void dirty_make_empty(dirty_set_t *ds);
int dirty_has_elements(dirty_set_t *ds);
int dirty_includes(dirty_set_t *ds, const int v);
void dirty_add(dirty_set_t *ds, const int v);
int dirty_remove(dirty_set_t *ds);
void dirty_destroy(dirty_set_t *ds);

@ The two possible implementations require different data in the set type.
However, the two implmementations have some common fields.

Field |n| is the domain size of the set: the set can contains entries
for 0 through $n-1$.

Field |num_used| counts the number of entries filled.  Before
initialization it is negative.  Afterward it is always non-negative.

@<Exported type definitions@>=
typedef struct {
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
@<Dictionary fields@>@;
#else /* FIFO */
@<FIFO fields@>@;
#endif
int n;
int num_used;
} dirty_set_t;

@*Skeleton implementations.

@<Subroutines@>=
dirty_set_t *
dirty_create(int n, int full, int seed)
{
	dirty_set_t *ds=new_of(dirty_set_t);
	errorif(n<0,"Dirty set must be over non-empty range: n=%d",n);
	ds->n = n;
	ds->num_used=-1;
#if DIRTY_SET==DIRTY_SET_SPLAY_ROOT
	@<Dictionary create@>@;
#else
	@<FIFO create@>@;
#endif
	if ( full ) { dirty_make_full(ds,seed); }
	else { dirty_make_empty(ds); }
	return ds;
}


@
@<Subroutines@>=
void
dirty_destroy(dirty_set_t *ds)
{
	if ( ds ) {
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary destroy@>@;
#else
	@<FIFO destroy@>@;
#endif
		ds->n = -1;
		free_mem(ds); @+ mem_deduct(sizeof(dirty_set_t));
	}
}


@
@<Subroutines@>=
void
dirty_add(dirty_set_t *ds, const int v)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary add@>@;
#else
	@<FIFO add@>@;
#endif
}

@
@<Subroutines@>=
int
dirty_includes(dirty_set_t *ds, const int v)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary includes |v|?@>@;
#else
	@<FIFO includes |v|?@>@;
#endif
}


@ If the set is empty, return $-1$.  That common behaviour is performed
here in the skeleton.  

@<Subroutines@>=
int
dirty_remove(dirty_set_t *ds)
{
	int return_value=-1;
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary remove@>@;
#else
	@<FIFO remove@>@;
#endif
return return_value;
}


@
@<Subroutines@>=
void
dirty_make_full(dirty_set_t *ds, int seed)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary make full@>@;
#else
	@<FIFO make full@>@;
#endif
	ds->num_used = ds->n;
}

@
@<Subroutines@>=
void
dirty_make_empty(dirty_set_t *ds)
{
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
	@<Dictionary make empty@>@;
#else
	@<FIFO make empty@>@;
#endif
	ds->num_used = 0;
}


@
@<Subroutines@>=
int
dirty_has_elements(dirty_set_t *ds)
{
	return ds->num_used > 0;
}



@*The dictionary implementation.
One possible implementation uses a splay-tree based dictionary.

@<Dictionary fields@>=
dict_t *dict;

@
@<Module headers required for the interface@>=
#if DIRTY_SET==DIRTY_SET_SPLAY_ROOT
#include "dict.h"
#endif

@ The creation routine just creates the dictionary.

@<Dictionary create@>=
ds->dict = dict_create(cmp_virtual_char,prn_virtual_char);


@ Instead of allocating a whole bunch of memory for the dirty set, I'll
just use a virtual pointer.  That is, the pointer stored in the
dictionary stores all the necessary information in its value, 
and it doesn't actually
point to anything.  This is safe because the pointer is never dereferenced.

In this case, the virtual pointers are just character offsets from byte 42.
This is mostly harmless.
@^virtual pointer@>
@^Adams, Douglas@>
@^mostly harmless@>
@^Earth, planet@>

The comparison function just subtracts the pointer values.  The semantics
of \CEE/ take care of the scaling.

The printing function prints the offset.

@d VIRTUAL_BASE ((char *)42)

@<Module subroutines@>=
#if DIRTY_SET == DIRTY_SET_SPLAY_ROOT
static int cmp_virtual_char(const void *a, const void *b);
static int 
cmp_virtual_char(const void *a, const void *b) 
{
	return (const char *)a - (const char *)b;
}

static void prn_virtual_char(void *a);
static void 
prn_virtual_char(void *a) 
{
	printf("%d",(const char *)a-VIRTUAL_BASE);
}
#endif

@ Destroying is easy.
@<Dictionary destroy@>=
if ( ds->dict ) dict_destroy(ds->dict,NULL);


@ To fill the universe, we just insert a representative
virtual pointer into the dictionary for each vertex.

@<Dictionary make full@>=
{
	int i;
	dict_t *d = ds->dict;
	for (i=0;i<n;i++) {
		dict_insert(d,VIRTUAL_BASE + i);
	}
}

@ To empty the set, we just delete all the elements from the dictionary.

@<Dictionary make empty@>=
dict_delete_all(ds->dict,NULL);

@ To add an entry, we try adding it to the dictionary.  The |dict_insert|
call answers non-zero if and only if the item was already present.

@<Dictionary add@>=
if (!dict_insert(ds->dict,VIRTUAL_BASE + v)) ds->num_used++;

@ Finding an element is similar.

(Ok, \CEE/ aficionados, I need the |!=NULL| fragment to convert the
pointer argument to an integer.)

@<Dictionary includes |v|?@>=
return dict_find(ds->dict,VIRTUAL_BASE+v) != NULL;

@ In removing, we need to distinguish the empty and non-empty cases.
If the dictionary is empty then |dict_delete_any| returns |NULL|.  
In that case we fall through and let the skeleton routine answer $-1$.

@<Dictionary remove@>=
{ void *vp = dict_delete_any(ds->dict,NULL);
if ( vp ) { ds->num_used--; return ((char *)vp)-VIRTUAL_BASE;}
}


@*The FIFO implementation.
An alternative implementation of the set of dirty edges is as a
first-in-first-out FIFO queue.
We implement this FIFO with three fields: 
array |queue| and pointer |head|.
@^FIFO@>

For any element $v$, there are three possible values for |queue[v]|.
If $v$ is not in the set, then |queue[v].next==NULL|.
If $v$ is the in the set and it is last, then 
|queue[v].next == &sentinel|, where |sentinel| is some dummy
cell that is never in the queue. 
If $v$ is the in the set and it is not last, then 
|queue[v].next| points to its successor in the list.

Value |head| is a pointer to the first element in the list, or |NULL|
if the list is empty.

Value |tail| is a pointer to the last element in the list, or |NULL|
if the list is empty.

I could have used integer indices, but then I either would have to offset
each index value by some positive constant or I would have to use some
value
other than 0 for the ``not present'' value.  I believe the pointer 
implementation  is simpler at the machine level, even though the
\CEE/ code for it is wordy.

@<FIFO fields@>=
dirty_queue_node_t *queue, *head, *tail;


@
@<Exported early type definitions@>=
typedef struct dirty_queue_node_s {
	struct dirty_queue_node_s *next;
} dirty_queue_node_t;

@  We will fill or empty the set later.
@<FIFO create@>=
ds->queue = new_arr_of(dirty_queue_node_t,n);

@
@<FIFO destroy@>=
free_mem(ds->queue); @+ mem_deduct(ds->n*sizeof(dirty_queue_node_t));
ds->head = NULL;
ds->tail = NULL;

@  Item |v| is present if and only if |ds->queue[v].next| is not |NULL|.
@<FIFO includes |v|?@>=
return ds->queue[v].next != NULL;

@  Conversely, item |v| is absent if and only if |ds->queue[v].next == NULL|.
We add it only if it is absent.
@<FIFO add@>=
{ dirty_queue_node_t *dq = ds->queue;
if ( dq[v].next == NULL ) { /* Add it at the end. */
	dq[v].next = &sentinel;		/* Need something non-|NULL|. */
	if ( ds->tail ) @+ ds->tail->next = dq+v; /* Connect to current tail. */
	else @+ ds->head = dq+v; /* List was empty. */
	ds->tail = dq+v;
	ds->num_used++;
}
}

@ Cell |sentinel| is a dummy cell.  It exists only so that we have
a cell with address different from |NULL| and which doesn't ever
participate in a set.

@<Module variables@>=
static dirty_queue_node_t sentinel;


@ The subtlety in removing an element
is that we need to save the answer in |return_value| 
before changing pointers.

@<FIFO remove@>=
if ( ds->head ) { /* Remove it from the beginning. */
	dirty_queue_node_t *old_head = ds->head;
	return_value = old_head - ds->queue;
	if ( old_head == ds->tail ) @+ ds->tail = ds->head  = NULL;
		/* List becomes empty. */
	else ds->head = old_head->next; /* Advance the pointer. */
	old_head->next = NULL; /* Signal that it is empty. */
	ds->num_used--;
}

@ Emptying out the queue is easy.   However, we split it into two
cases.  If the queue is not yet initialized or already quite full,
then we pretty much just blanket everything with |NULL| values.
If the queue is not filled very much, then we save time by removing
items one by one.

Recall that the the |num_used| field is negative before initialization.
The blanket filling code can be executed at
object initialization time, so it can't rely on |head| or |tail| having
been properly set.  In some cases scanning the entire array is
wasteful, but in the initial usage of this module I won't ever
call the make empty operation anyway.

In the one by one case, we just include the code from |@<FIFO remove@>|.
The compiler should optimize away the |if (ds->head)| code away
since it is always preceded (in control flow terms) by the
|while(ds->head)| loop test.

@<FIFO make empty@>=
if ( ds->num_used < 0 || ds->num_used >= ds->n / 2 ) { /* Blanket fill. */
	ds->head = ds->tail = NULL;
	{ int i;
	const int n = ds->n;
	dirty_queue_node_t *dq = ds->queue;
	for (i=0;i<n;i++) dq[i].next=NULL;
	}
} else { /* One by one. */
	int return_value; /* Needed by |@<FIFO remove@>|. */
	while (ds->head) { @+ @<FIFO remove@>@+ }
}

@  Filling the queue is matter of setting pointers, mostly.

However, filling it in random order forces an extra layer of complexity.
Each queue has its own random number generator.

@<FIFO fields@>=
prng_t *prng;

@
@<Module headers required for the interface@>=
#if DIRTY_SET==DIRTY_SET_FIFO
#include "prng.h"
#endif

@ We create the pseudo-random number stream at queue creation time.

We use the seed provided by the caller, but mix in some ``salt'' bits.
@^Alpha, DEC@>
@^DEC Alpha@>
@^21264@>

@<FIFO create@>=
ds->prng = prng_new(PRNG_DEFAULT,seed ^ 21264 ^ num_queues++);

@ The salt we mix in with the seed should change for different queues.
We simply keep a count of the number of queues allocated so far.
It increases monotonically: we don't decrement it when we destroy the
queue.

@<Module variables@>=
#if DIRTY_SET == DIRTY_SET_FIFO
static int num_queues=0;
#endif

@ We shouldn't forget to deallocate the generator.
@<FIFO destroy@>=
prng_free(ds->prng);

@ To fill the queue in random order, we first generate a random ordering
of the elements, and then insert them into the queue in that order.

@<FIFO make full@>=
{
const int n=ds->n;
prng_t *prng=ds->prng;
int i, *work=new_arr_of(int,n);

@<Write a random permutation into |work|@>@;
@<Write the |work| permutation into |ds|@>@;

free_mem(work); @+ mem_deduct(n*sizeof(int));
}


@  We must be careful to not repeat elements.
@<Write a random permutation into |work|@>=
for ( i=0; i<n; i++ ) work[i]=i; /* Identity permutation. */
for ( i=0; i<n; i++ ) { /* Find a random permutation. */
	const int next = prng_unif_int(prng,n-i);
	const int t = work[next];@+
	work[next] = work[n-1-i];@+
	work[n-1-i] = t;
}

@  This is just writing pointers that link properly.
@<Write the |work| permutation into |ds|@>=
{
dirty_queue_node_t *q = ds->queue;
ds->head = q+work[0];
ds->tail = q+work[n-1];
for (i=0;i<n-1;i++) {
	q[work[i]].next=q+work[i+1];
}
q[work[n-1]].next=&sentinel;
}



@*Testing.
Now let us get on with the business of testing dirty sets.

We'll use the command-line words as inputs to the sequence.

@(dirtytest.c@>=
#include <config.h>
#include "lkconfig.h"
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include "dirty.h"
#include "prng.h"

@<Test subroutines@>@;

int 
main(int argc, char **argv) 
{
	int i;
	prng_t *p = prng_new(PRNG_DEFAULT,21115); /* Linux is at version 2.1.114 as I write. */
	test_word("David Neto was here",p);
	for ( i=0; i<argc; i++ ) {
		test_word(argv[i],p);
	}
	test_word("Perl rocks!",p);
	return 0;
}

@ We use the characters in a word as the elements for the set.

@<Test subroutines@>=

void test_word(char const *word, prng_t *p);
void
test_word(char const *word, prng_t *p)
{
	int len = strlen(word), r, size;
	dirty_set_t *ds=dirty_create(256,0,prng_unif_int(p,32767));
	printf("Start empty: ");
	for ( r=0; r < len; r++ ) {
		while ( prng_unif_int(p,len) < len/2 ) @<Remove a character@>@;
		printf("+%c",word[r]);
		dirty_add(ds,(int)word[r]);
	}
	while ( dirty_has_elements(ds) ) @<Remove a character@>@;
	dirty_destroy(ds);

	printf("\nStart full:\n");
	size = len>26 ? 26 : len;
	ds=dirty_create(size,1,prng_unif_int(p,32767));
	for ( r=0;r<len;r++ )
	while ( dirty_has_elements(ds) ) { 
		@<Show integer contents@>@;
		@<Remove an integer@>@; 
	}
	@<Show integer contents@>@; 
	printf("\n\n");
}

@
@<Remove a character@>=
{
	const int ret = dirty_remove(ds);
	printf("-%c", ret==-1? ' ' : ret);
}

@
@<Remove an integer@>=
{
	const int ret = dirty_remove(ds);
	printf("\tExtract %c\n", ret==-1? ' ' : ret+'a');
}

@ Here's where we exercise |dirty_includes|.
@<Show integer contents@>=
{ int i;
	printf("\t\tSet includes: ");
	for (i=0;i<size; i++ ) {
		putchar( dirty_includes(ds,i) ? 'a'+i : ' ');
	}
	putchar('\n');
}


@*Index.
