% vim: set ts=4 sw=4:
\noindent Copyright \copyright 1994, 1995, 1996, 1997, 1998 David Neto
\smallskip

\noindent 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
\smallskip

\noindent 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
\smallskip

\noindent   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.
\smallskip

\noindent   
   You may contact David Neto via email at {\tt netod@@acm.org}, or with
   greater latency at
\smallskip
\noindent{\obeylines
     Department of Computer Science
     University of Toronto
     10 King's College Rd.
     Toronto, Ontario
     M5S 3G4
     Canada
}
\medskip


\noindent\hbox{}\hrule\hbox{}\penalty-1000
\vskip0.5cm\relax


@i webdefs.w
@i types.w

@*Milestones.
Whenever we make an improvement, we check to see if we've passed
a milestone in terms of percentage above a lower bound.
Of course, this is conditional a upon the user specifying a lower bound
on the command line.  

The outline of this module is as follows.

@c
#include "config.h"
#include "lkconfig.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "length.h"
#include "resource.h"
#include "milestone.h"

extern int verbose;
@<Module variables@>@;
@<Subroutines@>@;

@
We have the ability track multiple milestone tracks.  Each
has its own local state.

@(milestone.h@>=
typedef struct {
	int current;
	length_t target_value;
	length_t lower_bound_value;
	char *name;
	double *percentage;
	int num;
	double time;
	int tick_from;	/* From Which resource entry should we measure time? */
} milestone_state_t;


@ We check every tenth of a percentage point starting at 10\%, and going
down.  We go below zero because we might be checking against a too-high
estimate.

@<Module variables@>=
double default_percentages[]=
	{ .10,
	.099, .098, .097, .096, .095, .094, .093, .092, .091, .090,
	.089, .088, .087, .086, .085, .084, .083, .082, .081, .080,
	.079, .078, .077, .076, .075, .074, .073, .072, .071, .070,
	.069, .068, .067, .066, .065, .064, .063, .062, .061, .060,
	.059, .058, .057, .056, .055, .054, .053, .052, .051, .050,
	.049, .048, .047, .046, .045, .044, .043, .042, .041, .040,
	.039, .038, .037, .036, .035, .034, .033, .032, .031, .030,
	.029, .028, .027, .026, .025, .024, .023, .022, .021, .020,
	.019, .018, .017, .016, .015, .014, .013, .012, .011, .010,
	.009, .008, .007, .006, .005, .004, .003, .002, .001, .000,
   -.001,-.002,-.003,-.004,-.005,-.006,-.007,-.008,-.009,-.010,
   -.011,-.012,-.013,-.014,-.015,-.016,-.017,-.018,-.019,-.020,
   -.021,-.022,-.023,-.024,-.025,-.026,-.027,-.028,-.029,-.030,
   -.031,-.032,-.033,-.034,-.035,-.036,-.037,-.038,-.039,-.040,
   -.041,-.042,-.043,-.044,-.045,-.046,-.047,-.048,-.049,-.050,
   -.051,-.052,-.053,-.054,-.055,-.056,-.057,-.058,-.059,-.060,
   -.061,-.062,-.063,-.064,-.065,-.066,-.067,-.068,-.069,-.070,
   -.071,-.072,-.073,-.074,-.075,-.076,-.077,-.078,-.079,-.080,
   -.081,-.082,-.083,-.084,-.085,-.086,-.087,-.088,-.089,-.090,
   -.091,-.092,-.093,-.094,-.095,-.096,-.097,-.098,-.099,-.100
};
const int num_percentages=sizeof(default_percentages)/sizeof(double);

@ The first routine checks initializes the state.
@<Subroutines@>=
void 
milestone_initialize(milestone_state_t *ms, char *name, length_t lower_bound_value, int tick_from)
{
	if ( !ms ) return;
    ms->current=0;
    ms->name = name;
	ms->lower_bound_value = lower_bound_value;
    ms->percentage=default_percentages;
    ms->num=num_percentages;
    ms->target_value = lower_bound_value*(1+ms->percentage[ms->current]);
    ms->time=resource_user_tick_from(tick_from);
    ms->tick_from=tick_from;
}

@  This routine checks to see if we've gone under the current target.
If so, it prints a message for all the milestones we've passed.

@d max(x,y) ((x)<(y) ? (y) : (x))
@<Subroutines@>=
void
milestone_check_func(milestone_state_t *ms, length_t this_len)
{
	if ( !ms || !ms->name || ms->lower_bound_value < epsilon ) return;
	if ( this_len <= ms->target_value ) {
		const double this_time=resource_user_tick_from(ms->tick_from);
		char const kind[]="Intermediate";
		if ( ms->current == ms->num && this_len < ms->target_value ) {
			/* We're far below expectations, so always print. */
			const double pct = 
				100*(this_len - (double)ms->lower_bound_value)/
					(double)ms->lower_bound_value;
			@<Show a milestone at |pct| a given |kind|@>@;
			ms->time=this_time;
		} else { /* Take a percentage from a table. */
			while ( this_len <= ms->target_value && ms->current < ms->num ) {
				const double pct = 100*ms->percentage[ms->current];
				@<Show a milestone at |pct| a given |kind|@>@;
				ms->time=this_time;
				ms->current++;
				if ( ms->current < ms->num ) {
					ms->target_value 
						= ms->lower_bound_value*(1+ms->percentage[ms->current]);
				}
			}
		}
	}
}


@
@<Show a milestone at |pct| a given |kind|@>=
printf("Milestone:%s: length " length_t_native_spec
	" %5.2f%% %s %s after %.2f (+ %.2f) sec\n",
	kind,
	length_t_pcast(this_len),
	pct,
	(this_len >= ms->lower_bound_value ? "above" : "below"),
	ms->name,
	this_time,
	max(0,this_time-ms->time));
fflush(stdout);

@
@(milestone.h@>=
#define MILESTONE_VERBOSITY_LEVEL 25
#if MILESTONE_MAX_VERBOSE >= MILESTONE_VERBOSITY_LEVEL
#define milestone_check(ms,len) \
	do { \
		if ((ms) && (ms)->name && verbose >= MILESTONE_VERBOSITY_LEVEL) \
			milestone_check_func((ms),(len)); \
	} while(0)
#else
#define milestone_check(ms,len)
#endif

@
@<Module headers@>=
#include "resource.h"


@ We show where we started from, percentage-wise.

@d  epsilon (1e-5)

@<Subroutines@>=
void
milestone_show_initial(milestone_state_t *ms,length_t this_len)
{
#if MILESTONE_MAX_VERBOSE >= MILESTONE_VERBOSITY_LEVEL
if ( ms && ms->name && verbose >= MILESTONE_VERBOSITY_LEVEL && ms->lower_bound_value > epsilon ) {
	const double this_time=resource_user_tick_from(ms->tick_from);
	const double pct = 
		100*(this_len-(double)ms->lower_bound_value)/
			(double)ms->lower_bound_value;
	char const kind[]="Initial";
	@<Show a milestone at |pct| a given |kind|@>@;
}
#endif
}

@ Sometimes we are forced to output at someone else's request.
@<Subroutines@>=
void
milestone_show_request(milestone_state_t *ms,char *str, length_t this_len)
{
#if MILESTONE_MAX_VERBOSE >= MILESTONE_VERBOSITY_LEVEL
if ( str && ms && ms->name && verbose >= MILESTONE_VERBOSITY_LEVEL 
	&& ms->lower_bound_value > epsilon ) {
	const double this_time = resource_user_tick_from(ms->tick_from);
	const double pct = 
		100*(this_len-(double)ms->lower_bound_value)/(double)ms->lower_bound_value;
	char kind[1000];
	strcpy(kind,"Requested:");
	strncpy(kind+10,str,1000-10-1);
	@<Show a milestone at |pct| a given |kind|@>@;
}
#endif
}

@ We also show where we end up.
@<Subroutines@>=
void
milestone_show_final(milestone_state_t *ms, length_t this_len)
{
#if MILESTONE_MAX_VERBOSE >= MILESTONE_VERBOSITY_LEVEL
if ( ms && ms->name && verbose >= MILESTONE_VERBOSITY_LEVEL && ms->lower_bound_value > epsilon ) {
	double this_time=resource_user_tick_from(ms->tick_from);
	const double pct=100*(this_len-(double)ms->lower_bound_value)/(double)ms->lower_bound_value;
	char const kind[]="Final";
	@<Show a milestone at |pct| a given |kind|@>@;
}
#endif
}

@
@(milestone.h@>=
void milestone_initialize(milestone_state_t *ms, char *name, length_t bound_value,int tick_from);
void milestone_check_func(milestone_state_t *ms, length_t this_len);
void milestone_show_initial(milestone_state_t *ms,length_t this_len);
void milestone_show_request(milestone_state_t *ms,char *str, length_t this_len);
void milestone_show_final(milestone_state_t *ms, length_t this_len);

@*Revision log.
{\obeylines
$Log: milestone.w,v $
Revision 1.2  1998/10/02 19:13:21  neto
This gives more regular output, and actually seems to work right in
both TSP and matching.

Revision 1.1  1998/09/26 22:55:47  neto
Initial revision

}


@*Index.
